diff --git a/manifests/contextservice.yaml b/manifests/contextservice.yaml
index 96735bf5f89f682f31131c123ee9884a1becbfdb..df06c86b58849762277f5d99788e36b1933e9a69 100644
--- a/manifests/contextservice.yaml
+++ b/manifests/contextservice.yaml
@@ -41,6 +41,10 @@ spec:
           value: "nats"
         - name: LOG_LEVEL
           value: "INFO"
+        - name: ALLOW_EXPLICIT_ADD_DEVICE_TO_TOPOLOGY
+          value: "FALSE"
+        - name: ALLOW_EXPLICIT_ADD_LINK_TO_TOPOLOGY
+          value: "FALSE"
         envFrom:
         - secretRef:
             name: crdb-data
diff --git a/src/common/tests/MockServicerImpl_Context.py b/src/common/tests/MockServicerImpl_Context.py
index 55f87b7b0c03a7ae563dc10bd5e4964a07317c21..837445b5ac27f45ae224673912c7cafc3399c555 100644
--- a/src/common/tests/MockServicerImpl_Context.py
+++ b/src/common/tests/MockServicerImpl_Context.py
@@ -143,17 +143,34 @@ class MockServicerImpl_Context(ContextServiceServicer):
 
     def SetTopology(self, request: Topology, context : grpc.ServicerContext) -> TopologyId:
         LOGGER.debug('[SetTopology] request={:s}'.format(grpc_message_to_json_string(request)))
-        container_name = 'topology[{:s}]'.format(str(request.topology_id.context_id.context_uuid.uuid))
+        context_uuid = str(request.topology_id.context_id.context_uuid.uuid)
+        container_name = 'topology[{:s}]'.format(context_uuid)
         topology_uuid = request.topology_id.topology_uuid.uuid
         reply,_ = self._set(request, container_name, topology_uuid, 'topology_id', TOPIC_TOPOLOGY)
+
+        context_ = self.obj_db.get_entry('context', context_uuid, context)
+        for _topology_id in context_.topology_ids:
+            if _topology_id.topology_uuid.uuid == topology_uuid: break
+        else:
+            # topology not found, add it
+            context_.topology_ids.add().topology_uuid.uuid = topology_uuid
+
         LOGGER.debug('[SetTopology] reply={:s}'.format(grpc_message_to_json_string(reply)))
         return reply
 
     def RemoveTopology(self, request: TopologyId, context : grpc.ServicerContext) -> Empty:
         LOGGER.debug('[RemoveTopology] request={:s}'.format(grpc_message_to_json_string(request)))
-        container_name = 'topology[{:s}]'.format(str(request.context_id.context_uuid.uuid))
+        context_uuid = str(request.context_id.context_uuid.uuid)
+        container_name = 'topology[{:s}]'.format(context_uuid)
         topology_uuid = request.topology_uuid.uuid
         reply = self._del(request, container_name, topology_uuid, 'topology_id', TOPIC_TOPOLOGY, context)
+
+        context_ = self.obj_db.get_entry('context', context_uuid, context)
+        for _topology_id in context_.topology_ids:
+            if _topology_id.topology_uuid.uuid == topology_uuid:
+                context_.topology_ids.remove(_topology_id)
+                break
+
         LOGGER.debug('[RemoveTopology] reply={:s}'.format(grpc_message_to_json_string(reply)))
         return reply
 
@@ -368,17 +385,34 @@ class MockServicerImpl_Context(ContextServiceServicer):
 
     def SetSlice(self, request: Slice, context : grpc.ServicerContext) -> SliceId:
         LOGGER.debug('[SetSlice] request={:s}'.format(grpc_message_to_json_string(request)))
-        container_name = 'slice[{:s}]'.format(str(request.slice_id.context_id.context_uuid.uuid))
+        context_uuid = str(request.slice_id.context_id.context_uuid.uuid)
+        container_name = 'slice[{:s}]'.format(context_uuid)
         slice_uuid = request.slice_id.slice_uuid.uuid
         reply,_ = self._set(request, container_name, slice_uuid, 'slice_id', TOPIC_SLICE)
+
+        context_ = self.obj_db.get_entry('context', context_uuid, context)
+        for _slice_id in context_.slice_ids:
+            if _slice_id.slice_uuid.uuid == slice_uuid: break
+        else:
+            # slice not found, add it
+            context_.slice_ids.add().slice_uuid.uuid = slice_uuid
+
         LOGGER.debug('[SetSlice] reply={:s}'.format(grpc_message_to_json_string(reply)))
         return reply
 
     def RemoveSlice(self, request: SliceId, context : grpc.ServicerContext) -> Empty:
         LOGGER.debug('[RemoveSlice] request={:s}'.format(grpc_message_to_json_string(request)))
-        container_name = 'slice[{:s}]'.format(str(request.context_id.context_uuid.uuid))
+        context_uuid = str(request.slice_id.context_id.context_uuid.uuid)
+        container_name = 'slice[{:s}]'.format(context_uuid)
         slice_uuid = request.slice_uuid.uuid
         reply = self._del(request, container_name, slice_uuid, 'slice_id', TOPIC_SLICE, context)
+
+        context_ = self.obj_db.get_entry('context', context_uuid, context)
+        for _slice_id in context_.slice_ids:
+            if _slice_id.slice_uuid.uuid == slice_uuid:
+                context_.slice_ids.remove(_slice_id)
+                break
+
         LOGGER.debug('[RemoveSlice] reply={:s}'.format(grpc_message_to_json_string(reply)))
         return reply
 
@@ -443,17 +477,34 @@ class MockServicerImpl_Context(ContextServiceServicer):
 
     def SetService(self, request: Service, context : grpc.ServicerContext) -> ServiceId:
         LOGGER.debug('[SetService] request={:s}'.format(grpc_message_to_json_string(request)))
-        container_name = 'service[{:s}]'.format(str(request.service_id.context_id.context_uuid.uuid))
+        context_uuid = str(request.service_id.context_id.context_uuid.uuid)
+        container_name = 'service[{:s}]'.format(context_uuid)
         service_uuid = request.service_id.service_uuid.uuid
         reply,_ = self._set(request, container_name, service_uuid, 'service_id', TOPIC_SERVICE)
+
+        context_ = self.obj_db.get_entry('context', context_uuid, context)
+        for _service_id in context_.service_ids:
+            if _service_id.service_uuid.uuid == service_uuid: break
+        else:
+            # service not found, add it
+            context_.service_ids.add().service_uuid.uuid = service_uuid
+
         LOGGER.debug('[SetService] reply={:s}'.format(grpc_message_to_json_string(reply)))
         return reply
 
     def RemoveService(self, request: ServiceId, context : grpc.ServicerContext) -> Empty:
         LOGGER.debug('[RemoveService] request={:s}'.format(grpc_message_to_json_string(request)))
-        container_name = 'service[{:s}]'.format(str(request.context_id.context_uuid.uuid))
+        context_uuid = str(request.service_id.context_id.context_uuid.uuid)
+        container_name = 'service[{:s}]'.format(context_uuid)
         service_uuid = request.service_uuid.uuid
         reply = self._del(request, container_name, service_uuid, 'service_id', TOPIC_SERVICE, context)
+
+        context_ = self.obj_db.get_entry('context', context_uuid, context)
+        for _service_id in context_.service_ids:
+            if _service_id.service_uuid.uuid == service_uuid:
+                context_.service_ids.remove(_service_id)
+                break
+
         LOGGER.debug('[RemoveService] reply={:s}'.format(grpc_message_to_json_string(reply)))
         return reply
 
diff --git a/src/common/tools/context_queries/Topology.py b/src/common/tools/context_queries/Topology.py
index caf03ed0eb5271aa6e00a2c107a06f9e496d37dc..c90d59105ae6b0805a73e9a53c664e063703f73d 100644
--- a/src/common/tools/context_queries/Topology.py
+++ b/src/common/tools/context_queries/Topology.py
@@ -65,7 +65,7 @@ def get_topology(
 def get_topology_details(
         context_client : ContextClient, topology_uuid : str, context_uuid : str = DEFAULT_CONTEXT_NAME,
         rw_copy : bool = False
-    ) -> Optional[Topology]:
+    ) -> Optional[TopologyDetails]:
     try:
         # pylint: disable=no-member
         topology_id = TopologyId()
diff --git a/src/common/tools/descriptor/Loader.py b/src/common/tools/descriptor/Loader.py
index 916a73d300011c62fc008fc7437df8a71f6a9838..c5468c19ccc063387460ed7f7b28ee70c5f1d907 100644
--- a/src/common/tools/descriptor/Loader.py
+++ b/src/common/tools/descriptor/Loader.py
@@ -240,11 +240,16 @@ class DescriptorLoader:
         self._process_descr('slice',      'add',    self.__ctx_cli.SetSlice,      Slice,      self.__slices        )
         self._process_descr('connection', 'add',    self.__ctx_cli.SetConnection, Connection, self.__connections   )
 
-        # Update context and topology is useless:
-        # - devices and links are assigned to topologies automatically by Context component
-        # - topologies, services, and slices are assigned to contexts automatically by Context component
+        # By default the Context component automatically assigns devices and links to topologies based on their
+        # endpoints, and assigns topologies, services, and slices to contexts based on their identifiers.
+
+        # The following statement is useless; up to now, any use case requires assigning a topology, service, or
+        # slice to a different context.
         #self._process_descr('context',    'update', self.__ctx_cli.SetContext,    Context,    self.__contexts      )
-        #self._process_descr('topology',   'update', self.__ctx_cli.SetTopology,   Topology,   self.__topologies    )
+
+        # In some cases, it might be needed to assign devices and links to multiple topologies; the
+        # following statement performs that assignment.
+        self._process_descr('topology',   'update', self.__ctx_cli.SetTopology,   Topology,   self.__topologies    )
 
         #self.__ctx_cli.close()
 
@@ -271,12 +276,17 @@ class DescriptorLoader:
         self._process_descr('service',  'update', self.__svc_cli.UpdateService,   Service,  self.__services      )
         self._process_descr('slice',    'add',    self.__slc_cli.CreateSlice,     Slice,    self.__slices_add    )
         self._process_descr('slice',    'update', self.__slc_cli.UpdateSlice,     Slice,    self.__slices        )
-        
-        # Update context and topology is useless:
-        # - devices and links are assigned to topologies automatically by Context component
-        # - topologies, services, and slices are assigned to contexts automatically by Context component
+
+        # By default the Context component automatically assigns devices and links to topologies based on their
+        # endpoints, and assigns topologies, services, and slices to contexts based on their identifiers.
+
+        # The following statement is useless; up to now, any use case requires assigning a topology, service, or
+        # slice to a different context.
         #self._process_descr('context',  'update', self.__ctx_cli.SetContext,      Context,  self.__contexts      )
-        #self._process_descr('topology', 'update', self.__ctx_cli.SetTopology,     Topology, self.__topologies    )
+
+        # In some cases, it might be needed to assign devices and links to multiple topologies; the
+        # following statement performs that assignment.
+        self._process_descr('topology', 'update', self.__ctx_cli.SetTopology,     Topology, self.__topologies    )
 
         #self.__slc_cli.close()
         #self.__svc_cli.close()
diff --git a/src/context/Config.py b/src/context/Config.py
index 1549d9811aa5d1c193a44ad45d0d7773236c0612..770b1f065b93718c5428e895fae948d8e6ee1466 100644
--- a/src/context/Config.py
+++ b/src/context/Config.py
@@ -12,3 +12,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from common.Settings import get_setting
+
+TRUE_VALUES = {'Y', 'YES', 'T', 'TRUE', 'E', 'ENABLE', 'ENABLED'}
+def is_enabled(setting_name : str, default_value : bool) -> bool:
+    _is_enabled = get_setting(setting_name, default=None)
+    if _is_enabled is None: return default_value
+    str_is_enabled = str(_is_enabled).upper()
+    return str_is_enabled in TRUE_VALUES
+
+DEFAULT_VALUE = False
+ALLOW_EXPLICIT_ADD_DEVICE_TO_TOPOLOGY = is_enabled('ALLOW_EXPLICIT_ADD_DEVICE_TO_TOPOLOGY', DEFAULT_VALUE)
+ALLOW_EXPLICIT_ADD_LINK_TO_TOPOLOGY   = is_enabled('ALLOW_EXPLICIT_ADD_LINK_TO_TOPOLOGY',   DEFAULT_VALUE)
diff --git a/src/context/service/database/Topology.py b/src/context/service/database/Topology.py
index 1f0fb6c0b3c400d58ea83bc857e97bc50a1324a3..e8f99415509ee16e359d4737a2f4f41fed7f09a8 100644
--- a/src/context/service/database/Topology.py
+++ b/src/context/service/database/Topology.py
@@ -17,17 +17,20 @@ from sqlalchemy.dialects.postgresql import insert
 from sqlalchemy.engine import Engine
 from sqlalchemy.orm import Session, selectinload, sessionmaker
 from sqlalchemy_cockroachdb import run_transaction
-from typing import Dict, List, Optional
+from typing import Dict, List, Optional, Set
 from common.proto.context_pb2 import (
     ContextId, Empty, EventTypeEnum, Topology, TopologyDetails, TopologyId, TopologyIdList, TopologyList)
 from common.message_broker.MessageBroker import MessageBroker
 from common.method_wrappers.ServiceExceptions import NotFoundException
 from common.tools.object_factory.Context import json_context_id
 from common.tools.object_factory.Topology import json_topology_id
+from context.Config import ALLOW_EXPLICIT_ADD_DEVICE_TO_TOPOLOGY, ALLOW_EXPLICIT_ADD_LINK_TO_TOPOLOGY
 from .models.DeviceModel import DeviceModel
 from .models.LinkModel import LinkModel
 from .models.TopologyModel import TopologyDeviceModel, TopologyLinkModel, TopologyModel
 from .uuids.Context import context_get_uuid
+from .uuids.Device import device_get_uuid
+from .uuids.Link import link_get_uuid
 from .uuids.Topology import topology_get_uuid
 from .Events import notify_event_context, notify_event_topology
 
@@ -94,15 +97,40 @@ def topology_set(db_engine : Engine, messagebroker : MessageBroker, request : To
     if len(topology_name) == 0: topology_name = request.topology_id.topology_uuid.uuid
     context_uuid,topology_uuid = topology_get_uuid(request.topology_id, topology_name=topology_name, allow_random=True)
 
     # Ignore request.device_ids and request.link_ids. They are used for retrieving devices and links added into the
     # topology. Explicit addition into the topology is done automatically when creating the devices and links, based
     # on the topologies specified in the endpoints associated with the devices and links.
 
-    if len(request.device_ids) > 0:   # pragma: no cover
-        LOGGER.warning('Items in field "device_ids" ignored. This field is used for retrieval purposes only.')
+    related_devices : List[Dict] = list()
+    if ALLOW_EXPLICIT_ADD_DEVICE_TO_TOPOLOGY:
+        device_uuids : Set[str] = set()
+        for device_id in request.device_ids:
+            device_uuid = device_get_uuid(device_id)
+            if device_uuid not in device_uuids: continue
+            related_devices.append({'topology_uuid': topology_uuid, 'device_uuid': device_uuid})
+            device_uuids.add(device_uuid)
+    else:
+        if len(request.device_ids) > 0: # pragma: no cover
+            MSG = 'ALLOW_EXPLICIT_ADD_DEVICE_TO_TOPOLOGY={:s}; '.format(str(ALLOW_EXPLICIT_ADD_DEVICE_TO_TOPOLOGY))
+            MSG += 'Items in field "device_ids" ignored. This field is used for retrieval purposes only.'
+            LOGGER.warning(MSG)
 
-    if len(request.link_ids) > 0:    # pragma: no cover
-        LOGGER.warning('Items in field "link_ids" ignored. This field is used for retrieval purposes only.')
+    related_links : List[Dict] = list()
+    if ALLOW_EXPLICIT_ADD_LINK_TO_TOPOLOGY:
+        link_uuids : Set[str] = set()
+        for link_id in request.link_ids:
+            link_uuid = link_get_uuid(link_id)
+            if link_uuid not in link_uuids: continue
+            related_links.append({'topology_uuid': topology_uuid, 'link_uuid': link_uuid})
+            link_uuids.add(link_uuid)
+    else:
+        if len(request.link_ids) > 0:   # pragma: no cover
+            MSG = 'ALLOW_EXPLICIT_ADD_LINK_TO_TOPOLOGY={:s}; '.format(str(ALLOW_EXPLICIT_ADD_LINK_TO_TOPOLOGY))
+            MSG += 'Items in field "link_ids" ignored. This field is used for retrieval purposes only.'
+            LOGGER.warning(MSG)
 
     now = datetime.datetime.utcnow()
     topology_data = [{
@@ -124,7 +152,28 @@ def topology_set(db_engine : Engine, messagebroker : MessageBroker, request : To
         )
         stmt = stmt.returning(TopologyModel.created_at, TopologyModel.updated_at)
         created_at,updated_at = session.execute(stmt).fetchone()
-        return updated_at > created_at
+
+        updated = updated_at > created_at
+
+        updated_topology_device = False
+        if len(related_devices) > 0:
+            stmt = insert(TopologyDeviceModel).values(related_devices)
+            stmt = stmt.on_conflict_do_nothing(
+                index_elements=[TopologyDeviceModel.topology_uuid, TopologyDeviceModel.device_uuid]
+            )
+            topology_device_inserts = session.execute(stmt)
+            updated_topology_device = int(topology_device_inserts.rowcount) > 0
+
+        updated_topology_link = False
+        if len(related_links) > 0:
+            stmt = insert(TopologyLinkModel).values(related_links)
+            stmt = stmt.on_conflict_do_nothing(
+                index_elements=[TopologyLinkModel.topology_uuid, TopologyLinkModel.link_uuid]
+            )
+            topology_link_inserts = session.execute(stmt)
+            updated_topology_link = int(topology_link_inserts.rowcount) > 0
+
+        return updated or updated_topology_device or updated_topology_link
 
     updated = run_transaction(sessionmaker(bind=db_engine), callback)
     context_id = json_context_id(context_uuid)
@@ -94,15 +97,40 @@ def topology_set(db_engine : Engine, messagebroker : MessageBroker, request : To
     context_uuid,topology_uuid = topology_get_uuid(request.topology_id, topology_name=topology_name, allow_random=True)
 
-    # Ignore request.device_ids and request.link_ids. They are used for retrieving devices and links added into the
-    # topology. Explicit addition into the topology is done automatically when creating the devices and links, based
-    # on the topologies specified in the endpoints associated with the devices and links.
+    # By default, ignore request.device_ids and request.link_ids. They are used for retrieving
+    # devices and links added into the topology. Explicit addition into the topology is done
+    # automatically when creating the devices and links, based on the topologies specified in
+    # the endpoints associated with the devices and links.
+    # In some cases, it might be needed to add them explicitly; to allow that, activate flags
+    # ALLOW_EXPLICIT_ADD_DEVICE_TO_TOPOLOGY and/or ALLOW_EXPLICIT_ADD_LINK_TO_TOPOLOGY.
 